package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.visitor.GroupElementExceptionVisitor;
import de.fhdw.wtf.common.ast.visitor.GroupElementReturnVisitor;
import de.fhdw.wtf.common.ast.visitor.GroupElementVisitor;

/**
 * This class represents an element contained by a {@link Group}. It has to be either a {@link Group} by itself or a
 * <ClassType>.
 */
public interface GroupElement extends SyntaxObjectInterface {
	
	/**
	 * Returns the {@link Name} of this element.
	 * 
	 * @return : Name.
	 */
	Name getName();
	
	/**
	 * Accepts the sub types of {@link GroupElement} and implements the required functions.
	 * 
	 * @param visitor
	 *            is visitor that describes what has to be done for the particular {@link GroupElement}.
	 */
	void accept(GroupElementVisitor visitor);
	
	/**
	 * Visitor for subtypes of common.ast.GroupElement. Throws any Exception.
	 * 
	 * @param visitor
	 *            : Visitor.
	 * @param <X>
	 *            ReturnType
	 * @throws X
	 *             : Exception.
	 */
	<X extends Exception> void accept(GroupElementExceptionVisitor<X> visitor) throws X;
	
	/**
	 * Accepts a return visitor.
	 * 
	 * @param visitor
	 *            Visitor
	 * @param <X>
	 *            ReturnType
	 * @return X
	 */
	<X> X accept(GroupElementReturnVisitor<X> visitor);
	
}
