package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.SyntaxObject;
import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitorReturn;
import de.fhdw.wtf.common.token.Token;

/**
 * This class represents a modifier of a {@link ClassType}. A modifier comes to effect at the code generation.
 * 
 * 
 */
public abstract class ClassModifier extends SyntaxObject {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -1348944335686356604L;
	
	/**
	 * Constructor of {@link ClassModifier}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	protected ClassModifier(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	/**
	 * Accepts the sub types of {@link ClassModifier} and implements the required functions.
	 * 
	 * @param visitor
	 *            is visitor that describes what has to be done for the particular {@link ClassModifier}.
	 * @return Boolean
	 */
	public abstract boolean accept(ClassModifierVisitor visitor);
	
	/**
	 * Accepts a visitor with dynamic return type.
	 * 
	 * @param visitor
	 *            The Visitor to be accepted.
	 * @param <X>
	 *            ReturnType
	 * @return X.
	 */
	public abstract <X> X accept(ClassModifierVisitorReturn<X> visitor);
}
