package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitorReturn;
import de.fhdw.wtf.common.token.Token;

/**
 * A concrete {@link ClassModifier}. For {@link ClassType} with this modifier will be generated a visitor.
 */
public final class ClassModifierVisitable extends ClassModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 3480358484833493909L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 536835;
	
	/**
	 * Constructor of {@link ClassModifierVisitable}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private ClassModifierVisitable(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	@Override
	public String toString() {
		return de.fhdw.wtf.common.constants.parser.AstDescriptionConstants.MODIFIER_VISITABLE;
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof ClassModifierVisitable;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	/**
	 * Creates a {@link ClassModifierVisitable}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return The {@link ClassModifierVisitable}-Object
	 */
	public static ClassModifierVisitable create(final Token firstToken, final Token lastToken) {
		return new ClassModifierVisitable(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link ClassModifierVisitable}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @return The {@link ClassModifierVisitable}-Object
	 */
	public static ClassModifierVisitable create(final Token firstToken) {
		return new ClassModifierVisitable(firstToken, null);
	}
	
	@Override
	public boolean accept(final ClassModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final ClassModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
	
}
