package de.fhdw.wtf.common.ast.type;

import java.io.Serializable;

import de.fhdw.wtf.common.ast.DatabaseIDState;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitor;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitorException;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitorReturnException;

/**
 * This interface represents an abstract state of {@link TypeProxy}.
 */
public interface TypeProxyState extends Comparable<TypeProxyState>, Serializable {
	
	// compareTo ist wie folgt programmiert: InvalidState ByName ByType
	
	/**
	 * Accept-Method from Visitor-Pattern.
	 * 
	 * @param v
	 *            visitor
	 */
	void accept(TypeProxyStateVisitor v);
	
	/**
	 * Accept-Method from Visitor-Pattern.
	 * 
	 * @param <X>
	 *            Returntype
	 * @param v
	 *            visitor
	 * @return X
	 */
	<X> X accept(TypeProxyStateVisitorReturn<X> v);
	
	/**
	 * Accept-Method from Visitor-Pattern.
	 * 
	 * @param <Y>
	 *            ExceptionType
	 * @param v
	 *            visitor
	 * @throws Y
	 *             Exception
	 * 
	 */
	<Y extends Exception> void accept(TypeProxyStateVisitorException<Y> v) throws Y;
	
	/**
	 * Accept-Method from Visitor-Pattern.
	 * 
	 * @param <X>
	 *            Returntype
	 * @param <Y>
	 *            ExceptionType
	 * @param v
	 *            visitor
	 * @return X
	 * @throws Y
	 *             Exception
	 */
	<X, Y extends Exception> X accept(TypeProxyStateVisitorReturnException<X, Y> v) throws Y;
	
	@Override
	boolean equals(Object obj);
	
	/**
	 * Returns a string representation of the type behind this proxy. See
	 * {@link de.fhdw.wtf.common.ast.type.Type#getTypeString()}.
	 * 
	 * @return A string representation of the type behind this proxy.
	 */
	String getTypeString();
	
	/**
	 * Returns the database ID of the underlying type.
	 * 
	 * @return The database ID of the underlying type.
	 */
	DatabaseIDState getTypeId();
	
	/**
	 * Sets the database ID of the underlying type.
	 * 
	 * @param typeId
	 *            The new database ID.
	 */
	void setTypeId(final DatabaseIDState typeId);
}
