package de.fhdw.wtf.common.token;

/**
 * This token contains one identifier - e.g. a class identifier.
 * 
 */
public final class IdentifierToken extends MultiCharToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -2521207444154441984L;
	
	/**
	 * Private Constructor for IdentifierToken. Use create(...)-Factory instead!
	 * 
	 * @param identifier
	 *            initial identifier string
	 * @param position
	 *            start position of this token in the original input.
	 */
	private IdentifierToken(final String identifier, final Position position) {
		super(identifier, position);
	}
	
	/**
	 * Factory for a {@link IdentifierToken}.
	 * 
	 * @param identifier
	 *            initial identifier string
	 * @param position
	 *            start position of this token in the original input.
	 * @return a new instance of this token
	 */
	public static IdentifierToken create(final String identifier, final Position position) {
		return new IdentifierToken(identifier, position);
	}
	
	@Override
	public boolean isIdentifierToken() {
		return true;
	}
	
	/**
	 * Identifier as string representation.
	 * 
	 * @return identifier represented by this token
	 */
	public String getIdentifier() {
		return this.getTokenString();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		if (t.isIdentifierToken()) {
			final IdentifierToken ct = (IdentifierToken) t;
			return this.getIdentifier().equals(ct.getIdentifier());
		}
		return false;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return this.getIdentifier().hashCode();
	}
	
	@Override
	public boolean equals(final Object o) {
		if (o instanceof IdentifierToken) {
			return super.equals(o);
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}
	
}
