package de.fhdw.wtf.common.token;

/**
 * This token contains at least one whitespace.
 */
public class WhitespaceToken extends MultiCharToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 7922055796052935145L;
	
	/**
	 * Protected Constructor for WhitespaceToken. Use create(...)-Factory instead!
	 * 
	 * @param whitespace
	 *            initial whitespace string
	 * @param position
	 *            start position of this whitespace in the original input
	 */
	protected WhitespaceToken(final String whitespace, final Position position) {
		super(whitespace, position);
	}
	
	/**
	 * Factory for a {@link WhitespaceToken}.
	 * 
	 * @param whitespace
	 *            initial whitespace string
	 * @param position
	 *            start position of this comment in the original input
	 * @return a new instance of this token
	 */
	public static WhitespaceToken create(final String whitespace, final Position position) {
		return new WhitespaceToken(whitespace, position);
	}
	
	@Override
	public boolean isWhitespaceToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		if (t instanceof WhitespaceToken) {
			final WhitespaceToken ct = (WhitespaceToken) t;
			return this.getWhitespace().equals(ct.getWhitespace());
		}
		return false;
	}
	
	/**
	 * Whitespace as string representation.
	 * 
	 * @return whitespace represented by this token
	 */
	public String getWhitespace() {
		return this.getTokenString();
	}
	
	@Override
	protected int hashCodeTemplate() {
		return this.getWhitespace().hashCode();
	}
	
}
