package de.fhdw.wtf.common.token.keywords;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the "findable"-modifier as a token in WTF.
 */
public final class FindableToken extends AttributeModifierToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 336319822101900195L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 1233856;
	
	/**
	 * Defines constant "findable" for {@link FindableToken}.
	 */
	private static final String TEXTUAL_REPRESENTATION = "findable";
	
	/**
	 * Private constructor for {@link FindableToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of an {@link FindableToken} in the original input.
	 */
	private FindableToken(final Position position) {
		super(position);
	}
	
	/**
	 * Factory method for {@link FindableToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link AttributeModifierToken} in the original input.
	 * @return a new instance of this {@link AttributeModifierToken}.
	 */
	public static FindableToken create(final Position position) {
		return new FindableToken(position);
	}
	
	@Override
	public int getLength() {
		return FindableToken.TEXTUAL_REPRESENTATION.length();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isFindableToken();
	}
	
	@Override
	public String stringRepresentation() {
		return FindableToken.TEXTUAL_REPRESENTATION;
	}
	
	@Override
	public boolean isFindableToken() {
		return true;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
