package de.fhdw.wtf.common.token.keywords;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the transient-modifier as token in WTF.
 */
public final class TransientToken extends AttributeModifierToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -5650264699254593001L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 543545;
	
	/**
	 * Defines constant "transient" for {@link TransientToken}.
	 */
	private static final String TEXTUAL_REPRESENTATION = "transient";
	
	/**
	 * Private constructor for {@link TransientToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the {@link AttributeModifierToken} <prior> in the original input.
	 */
	private TransientToken(final Position position) {
		super(position);
	}
	
	/**
	 * Factory method for {@link TransientToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link AttributeModifierToken} in the original input.
	 * @return a new instance of this {@link AttributeModifierToken}.
	 */
	public static TransientToken create(final Position position) {
		return new TransientToken(position);
	}
	
	@Override
	public int getLength() {
		return TransientToken.TEXTUAL_REPRESENTATION.length();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isTransientToken();
	}
	
	@Override
	public String stringRepresentation() {
		return TransientToken.TEXTUAL_REPRESENTATION;
	}
	
	@Override
	public boolean isTransientToken() {
		return true;
	}
	
	@Override
	public boolean isClassModifierToken() {
		return true;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
