package de.fhdw.wtf.common.token.symbols;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the square bracket open symbol '[' as a token.
 */
public final class SquareBracketOpenToken extends SymbolToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 6989343494151304597L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 563623;
	
	/**
	 * Private constructor for {@link SquareBracketOpenToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the symbol '[' in the original input.
	 */
	private SquareBracketOpenToken(final Position position) {
		super(position, SymbolConstants.SQUAREBRACKETOPEN);
	}
	
	/**
	 * Factory method for {@link SquareBracketOpenToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link SymbolToken} in the original input.
	 * @return a new instance of this {@link SymbolToken}.
	 */
	public static SquareBracketOpenToken create(final Position position) {
		return new SquareBracketOpenToken(position);
	}
	
	@Override
	public boolean isSquareBracketOpenToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isSquareBracketOpenToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymbolConstants.SQUAREBRACKETOPEN;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
}
