package de.fhdw.wtf.walker.walker;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.ConstructorOrOperation;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.exception.walker.TaskException;
import de.fhdw.wtf.common.task.DependencyTask;
import de.fhdw.wtf.common.task.TaskExecutor;

/**
 * Walker through the WTF model.
 */
public abstract class WalkerTask extends DependencyTask {
	
	/**
	 * Constructor for {@link WalkerTask}.
	 * 
	 * @param taskmanager
	 *            Taskmanager
	 */
	public WalkerTask(final TaskExecutor taskmanager) {
		super(taskmanager);
	}
	
	/**
	 * This handle will be called with each {@link ClassType} in a given model ONCE PER CLASSTYPE!
	 * 
	 * @param c
	 *            ClassType to be handled
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void handleClass(ClassType c) throws TaskException;
	
	/**
	 * This handle will be called with each {@link Group} in a given model ONCE PER GROUP!
	 * 
	 * @param g
	 *            Group to be handled
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void handleGroup(Group g) throws TaskException;
	
	/**
	 * This handle will be called with each {@link Attribute} in a given model ONCE PER ATTRIBUTE!
	 * 
	 * @param a
	 *            Attribute to be handled
	 * @param owner
	 *            Class with the attribute
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void handleAttribute(Attribute a, ClassType owner) throws TaskException;
	
	/**
	 * This handle will be called with each {@link ConstructorOrOperation} in a given model ONCE PER
	 * CONSTRUCTOROROPERATION!
	 * 
	 * @param coo
	 *            ConstructorOrOperation to be handled
	 * @param owner
	 *            Class with the Constructor or Operation
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void handleConstructorOrOperation(ConstructorOrOperation coo, ClassType owner) throws TaskException;
	
	/**
	 * This operation will be called ONCE at the end of the task!
	 * 
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void finalizeTask() throws TaskException;
	
	/**
	 * This operation will be called ONCE at the begin of the task!
	 * 
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void beginTask() throws TaskException;
	
}
